/* Flaechenberechnung fuer achsparalleles Rechteck.
   Dieses Beispiel demonstriert die Verwendung von struct.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <math.h>


/** Koordinaten fuer einen Punkt.
*/
typedef struct {
    double    x;  /**< x-Koordinate. */
    double    y;  /**< y-Koordinate. */
} PUNKT;


/** Achsenparalleles Rechteck.
*/
typedef struct {
    PUNKT     lu; /**< Eckpunkt links unten. */
    PUNKT     ro; /**< Eckpunkt rechts oben. */
} RECHTECK;


/** Konstantes Rechteck, gegeben durch zwei Punkte.
*/
const RECHTECK re = { { 1.0, 2.0 }, { 3.0, 4.0} };


/** Hauptprogramm.
    @return 0 bei Erfolg, alle anderen Werte bedeuten Fehler.
*/
int
main(void)
{
    double    breite;
    double    hoehe;
    double    flaeche;

    /*  Direkter Zugriff.
        re.ro liefert die Komponente ro des RECHTECK re, das Resultat
        ist ein PUNKT.
        Das angehangene .x liefert hiervon die Komponente x, das
        Resultat ist der double-Wert mit der x-Koordinate.
    */
    breite = re.ro.x - re.lu.x;
    hoehe  = re.ro.y - re.lu.y;
    /* fabs() bildet Betrag. */
    flaeche = fabs(breite * hoehe);
    printf("Flaeche: %lg\n", flaeche);
    return 0;
}

/* vim: set ai sw=4 ts=4 expandtab : */
