/** @file   ex018.c Beispielprogramm fuer Zerlegung des Quelltextes in
    mehrere Module.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#include <stdio.h>
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif

#include "he-read.h"
#include "he-geom.h"



/** Quadrat: Abfrage der Seitenlaenge, Berechnung des Flaecheninhaltes.
    @return 1 bei Erfolg, 0 bei Fehlern.
*/
static
int
quadrat(void)
{
    double     fl     = 0.0;  /* Flaeche */
    double     slgt   = 0.0;  /* Seitenlaenge */
    int        res    = 0;    /* Ergebnis der Funktion */

    /* Nur wenn ein double-Wert eingegeben wurde ... */
    if (0 < hsm_et_read_double("Seitenlaenge: ", &slgt)) {
        /* ... und dieser Wert nicht negativ ist, ... */
        if (0.0 <= slgt) {
            /* ... koennen wir die Flaeche berechnen und Erfolg vermerken */
            fl = hsm_et_geom_quadrat_flaeche(slgt);
            printf("Flaeche: %lg\n", fl);
            res = 1;
        }
        else {
            fputs(
                "FEHLER: Negative Seitenlaengen sind nicht zulaessig!\n", stderr
            );
            fflush(stderr);
        }
    }
    return res;
}



/** Rechteck: Abfrage der Seitenlaengen, Berechnung des Flaecheninhaltes.
    @return 1 bei Erfolg, 0 bei Fehlern.
*/
static
int
rechteck(void)
{

    double     fl     = 0.0;  /* Flaeche */
    double     slga   = 0.0;  /* Seitenlaenge */
    double     slgb   = 0.0;  /* Andere Seitenlaenge */
    int        res    = 0;    /* Ergebnis der Funktion */

    /* Nur wenn ein double-Wert eingegeben wurde ... */
    if (0 < hsm_et_read_double("Seitenlaenge: ", &slga)) {
        /* ... und dieser Wert nicht negativ ist, ... */
        if (0.0 <= slga) {
            /* ... lesen wir die zweite Seitenlaenge ein... */
            if (0 < hsm_et_read_double("Andere Seitenlaenge: ", &slgb)) {
                /* ... testen, ob diese nicht negativ ist ... */
                if (0.0 <= slgb) {
                    /* ... berechnen die Flaeche und geben das Ergebnis aus. */
                    fl = hsm_et_geom_rechteck_flaeche(slga, slgb);
                    printf("Flaeche: %lg\n", fl);
                    res = 1;
                }
                else {
                    fputs(
                        "FEHLER: Negative Seitenlaengen sind nicht zulaessig!\n",
                        stderr
                    );
                    fflush(stderr);
                }
            }
        }
        else {
            fputs(
                "FEHLER: Negative Seitenlaengen sind nicht zulaessig!\n", stderr
            );
            fflush(stderr);
        }
    }
    return res;
}



/** Hauptprogramm.
        @return 0 bei Erfolg, alle anderen Werte weisen auf Fehler hin.
*/
int
main(void)
{
    int   excode  = 1;    /* Exit-Status-Code des Programmes */
    int   aktion  = 0;    /* Auszufuehrende Aktion */

    fputs("1=Quadratflaeche   2=Rechteckflaeche\n", stdout);
    if (0 < hsm_et_read_int("Aktion: ", &aktion)) {
        switch (aktion) {
            case 1: {
                if (0 < quadrat()) {
                    excode = 0;
                }
            } break;
            case 2: {
                if (0 < rechteck()) {
                    excode = 0;
                }
            } break;
            default: {
                fputs("", stderr);
                fflush(stderr);
            } break;
        }
    }
    exit(excode); return excode;
}

/* vim: set ai sw=4 ts=4 expandtab : */
