/** @file ex023.c Demo-Programm fuer Schreiben einer Text-Datei.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#ifdef _WIN32
/** Benutzung von fopen() unter Windows zulassen. */
#define _CRT_SECURE_NO_WARNINGS 1
#endif

#include <stdio.h>

#include <stdlib.h>
#ifndef _WIN32
#include <unistd.h>
#endif
#ifdef _WIN32
#include <process.h>
#endif


/** Dateiname der zu schreibenden Datei.
*/
#ifdef _WIN32
static const char filename[] = { "C:\\Temp\\testfile.txt" };
#else
static const char filename[] = { "testfile.txt" };
#endif


/** Hauptprogramm.
    @return 0 bei Erfolg, alle anderen Werte bedeuten Fehler.
*/
int
main(void)
{
    FILE *outf    = NULL;         /* Datei-Zeiger */
    int   exc     = EXIT_FAILURE; /* Exit-Code des Programmes. */
    int   i       = 5;            /* Variable fuer Testzwecke */

    outf = fopen(filename, "w");
    if (NULL != outf) {

        /* Erfolg im Rueckgabecode vermerken */
        exc = EXIT_SUCCESS;

        /* Falls Schreiben der Textzeile fehlschlaegt, Misserfolg */
        if (EOF == fputs("Dies ist ein Test-String", outf)) {
            exc = EXIT_FAILURE;
        }

        /* Falls Schreiben des Newline fehlschlaegt, Misserfolg */
        if (EOF == fputc('\n', outf)) {
            exc = EXIT_FAILURE;
        }

        /* Falls Schreiben der formatierten Ausgabe fehlschlaegt, Misserfolg */
        if (0 > fprintf(outf, "Der Wert der Variablen ist %d\n", i)) {
            exc = EXIT_FAILURE;
        }

        /* Falls Schliessen der Datei fehlschlaegt, Misserfolg */
        if (0 != fclose(outf)) {
            exc = EXIT_FAILURE;
        }
        outf = NULL;
    }
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
