/** @file   ex040.c Demo-Programm fuer Datei-Attribute auf Windows-Systmen.

    ex040 <Dateiname>
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

/** Hauptprogramm.
    @param  argc    Anzahl der Kommandozeilenargumente.
    @param  argv    Feld mit Kommandozeilenargumenten.
    @return 0 bei Erfolg, alle anderen Werte deuten auf Fehler hin.
*/
int
main(int argc, char *argv[])
{
    struct __stat64   stbuf;
    struct tm         timebuf;
    int               exc     =   EXIT_FAILURE;

    /* Es muss ein Argument angegeben sein: Dateiname */
    if (2 != argc) {
        /* !!! Falsche Argumente-Anzahl */
        goto finished;
    }

    /* Versuche, Datei-Attribute abzufragen */
    if (0 != _stat64(argv[1], &stbuf)) {
        /* !!! Abfrage der Dateiattribute fehlgeschlagen */
        goto finished;
    }

    /* Bis jetzt erfolgreich */
    exc = EXIT_SUCCESS;

    /* Allgemeine Informationen ueber Datei ausgeben */
    printf("Datei-Name:             %s\n",   argv[1]);
    printf("Groesse:                %I64u\n",  (uintmax_t)(stbuf.st_size));

    /* Dateityp ermitteln und ausgeben */
    switch(stbuf.st_mode & _S_IFMT) {
        case _S_IFREG: {
            printf("Dateityp:               Regulaere Datei\n");
        } break;
        case _S_IFDIR: {
            printf("Dateityp:               Verzeichnis\n");
        } break;
        case _S_IFCHR: {
            printf("Dateityp:               Byteorientiertes Geraet\n");
        } break;
        case _S_IFIFO: {
            printf("Dateityp:               FIFO (benannte Pipe)\n");
        } break;
        default: {
            printf("Dateityp:               Unbekannt\n");
            exc = EXIT_FAILURE;
        } break;
    }

    /* Eigentuemer ausgeben */
    printf("Eigentuemer-UID:        %d\n",  (int)(stbuf.st_uid));
    printf("Eigentuemer-GID:        %d\n",  (int)(stbuf.st_gid));
    
    printf(
        "Berechtigungen:         %c%c%c\n",
        ((0 != (stbuf.st_mode & _S_IREAD))  ? 'r' : '-'),
        ((0 != (stbuf.st_mode & _S_IWRITE)) ? 'w' : '-'),
        ((0 != (stbuf.st_mode & _S_IEXEC))  ? 'x' : '-')
    );

    /* Weitere Informationen ausgeben */
    printf("Link-Anzahl:            %d\n",    (int)(stbuf.st_nlink));
    printf("Device-ID:              %I64u\n", (uintmax_t)(stbuf.st_dev));
    printf("Rel. Device-ID:         %I64u\n", (uintmax_t)(stbuf.st_rdev));
    printf("Inode-Nummer:           %I64u\n", (uintmax_t)(stbuf.st_ino));

    /* Zeitpunkt ausgeben, wann Datei angelegt wurde */
    if (0 == _localtime64_s(&timebuf, &(stbuf.st_ctime))) {
        printf(
            "Erzeugungszeitpunkt:    %04d-%02d-%02d %02d:%02d:%02d\n",
            (1900 + timebuf.tm_year),
            (1 + timebuf.tm_mon),
            timebuf.tm_mday,
            timebuf.tm_hour,
            timebuf.tm_min,
            timebuf.tm_sec
        );
    }
    else {
        exc = EXIT_FAILURE;
    }

    /* Zeitpunkt der letzten Aenderung ausgeben */
    if (0 == _localtime64_s(&timebuf, &(stbuf.st_mtime))) {
        printf("Modifikationszeitpunkt: %04d-%02d-%02d %02d:%02d:%02d\n",
            (1900 + timebuf.tm_year),
            (1 + timebuf.tm_mon),
            timebuf.tm_mday,
            timebuf.tm_hour,
            timebuf.tm_min,
            timebuf.tm_sec
        );
    }
    else {
        exc = EXIT_FAILURE;
    }

    /* Zeitpunkt des letzten Zugriffes auf Datei ausgeben */
    if (0 == _localtime64_s(&timebuf, &(stbuf.st_atime))) {
        printf("Zugriffszeitpunkt:      %04d-%02d-%02d %02d:%02d:%02d\n",
            (1900 + timebuf.tm_year),
            (1 + timebuf.tm_mon),
            timebuf.tm_mday,
            timebuf.tm_hour,
            timebuf.tm_min,
            timebuf.tm_sec
        );
    }
    else {
        exc = EXIT_FAILURE;
    }

finished:
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
