/** @file   ex065.c Beispielprogramm binaeres Lesen.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS 1
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <stdint.h>
#include <limits.h>

#include "he-read.h"


/** Name der zu schreibenden Datei.
*/
static const char filename[] = {
#ifdef _WIN32
"C:\\Temp\\ex064.dat"
#else
"ex064.dat"
#endif
};


/** Hauptprogramm.
        @return 0 bei Erfolg.
*/
int
main(void)
{
    double    *werte  =   NULL;   /* Dynamischer Speicher fuer Werte */
    ???       *inpf   =   NULL;   /* Eingabedatei */
    int        num    =   0;      /* Anzahl der double-Werte */
    int        exc    =   1;      /* Exit-Status-Code */
    int        i;                 /* Index, aktuell auszugebender Wert */

    /* Eingabedatei zum binaeren Lesen oeffnen */
    ??? = fopen(???, ???);
    if (NULL == inpf) {
        fprintf(stderr, "FEHLER: Datei \"%s\" nicht geoeffnet!\n", filename);
        fflush(stderr);
        goto finished;
    }

    /* Anzahl der Datensaetze einlesen */
    if (1 != fread(&num, sizeof(???), ???, inpf)) {
        i = errno;
        fputs("FEHLER: Anzahl der Datensaetze nicht gelesen!\n", stderr);
        fflush(stderr);
        errno = i;
        perror("fread");
        goto finished;
    }

    /* Eingelesene Anzahl pruefen (positiv, kein Ueberlauf) */
    if (1 > num) {
        fputs("FEHLER: Anzahl der Datensaetze muss mindestens 1 sein!\n",stderr);
        fflush(stderr);
        goto finished;
    }
    if((SIZE_MAX / sizeof(double)) < (size_t)num) {
        fputs("FEHLER: Anzahl der Datensaetze zu gross!\n", stderr);
        fflush(stderr);
        goto finished;
    }

    /* Speicher dynamisch anfordern */
    werte = (double *)calloc(sizeof(???), ???);
    if (NULL == werte) {
        fputs("FEHLER: Kein Speicher verfuegbar!\n", stderr);
        fflush(stderr);
        goto finished;
    }

    /* Werte einlesen */
    if ((size_t)num != fread(werte, sizeof(???), (size_t)???, inpf)) {
        i = errno;
        fputs("FEHLER: Einlesen der Werte fehlgeschlagen!\n", stderr);
        fflush(stderr);
        errno = i;
        perror("fread");
        goto finished;
    }

    /* Erfolg vermerken */
    exc = 0;

    /* Werte ausgeben */
    for (i = 0; i < num; i++) {
        printf("Wert %d: %lg\n", (i+1), werte[i]);
    }

    /*	Aufraeumarbeiten und Programmende.
	----------------------------------
    */
finished:
    /*	Dynamisch allozierten Speicher fuer Werte freigeben, falls erhalten.
    */
    if (NULL != werte) {
        free(werte);
    }
    /*	Eingabedatei schliessen, falls erfolgreich geoeffnet.
    */
    if (NULL != inpf) {
        fclose(inpf);
    }
    /*	Programm verlassen, Exit-Status-Code zurueckgeben.
    */
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
