/* Testprogram zur Demonstration von Gleitkomma-Exceptions unter Windows
   Mit z.B. folgenden Eingaben ausprobieren:

   1.5e200
   1.8e-200

*/
/* Copyright (C) 2015-2017 - HS Schmalkalden. All rights reserved. */

#define _CRT_SECURE_NO_WARNINGS 1
#include <stdio.h>

#define _USE_MATH_DEFINES       1
#include <math.h>
#include <float.h>



/**     Flaechenberechnung fuer Quadrat.
        @param  a   Seitenlaenge des Quadrates.
        @return Flaecheninhalt des Quadrates bei Erfolg, -1.0 bei Fehler
        (negative Seitenlaenge).
*/
double
quadratflaeche(double a);


/**     Gleitkomma-Exceptions.
*/
unsigned int        flp_exc     =   0;

/**     Maske, welche Exceptions geprueft werden sollen.
*/
const unsigned int  flp_mask    =
_SW_ZERODIVIDE | _SW_INVALID | _SW_OVERFLOW | _SW_UNDERFLOW | _SW_DENORMAL;



/** Hauptprogramm.
    @return 0 bei Erfolg, 1 bei Fehler.
*/
int
main(void)
{
    int       exitcode = 1;   /* Exit-Code, 0 bei Erfolg, 1 bei Fehler */
    double    slaenge;        /* Seitenlaenge */
    double    flaeche;

    printf("Bitte Seitenlaenge fuer Quadrat angeben: ");
    scanf("%lg", &slaenge);
    if (0.0 <= slaenge) {
        (void)_clearfp();
        flaeche = quadratflaeche(slaenge);
        flp_exc = _statusfp();
        (void)_clearfp();
        if (0 != (flp_exc & flp_mask)) {
            printf("Es ist eine Gleitkomma-Ausnahme aufgetreten.\n");
            if (0 != (_SW_ZERODIVIDE & flp_exc)) {
                printf("Division durch 0!\n");
            }
            if (0 != (_SW_INVALID & flp_exc)) {
                printf("Ungueltiger Operand!\n");
            }
            if (0 != (_SW_OVERFLOW & flp_exc)) {
                printf("Zahlenbereichsueberlauf!\n");
            }
            if (0 != (_SW_UNDERFLOW & flp_exc)) {
                printf("Zahlenbereichsunterlauf!\n");
            }
            if (0 != (_SW_DENORMAL & flp_exc)) {
                printf("Denormalisierte Darstellung mit Genauigkeitsverlust!\n");
            }
        }
        else {
            printf("Flaecheninhalt: %lg\n", flaeche);
        }
        exitcode = 0;
    }
    else {
        printf("FEHLER: Seitenlaenge darf nicht negativ sein!\n");
    }
    return exitcode;
}



double
quadratflaeche(double a)
{
    double    ergebnis    =   -1.0;

    if (0.0 <= a) {
        ergebnis = a * a;
    }
    return    ergebnis;
}

/* vim: set ai sw=4 ts=4 expandtab : */
