/** @file ex025.c Demo-Programm fuer Schreiben einer Text-Datei,
	portabel mit he-file Modul.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#include <stdio.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif

#include "he-file.h"


/** Dateiname der zu schreibenden Datei.
*/
#if HE_ON_WINDOWS
static const char filename[] = { "C:\\Temp\\C\\testfile.txt" };
#else
static const char filename[] = { "testfile.txt" };
#endif


/** Hauptprogramm.
	@return 0 bei Erfolg, alle anderen Werte bedeuten Fehler.
*/
int
main(void)
{
	FILE *outf   = NULL;          /* Datei-Zeiger */
	int   exc    = EXIT_FAILURE;  /* Exit-Code des Programmes. */
	int   i      = 5;             /* Variable, wird ausgegeben. */

	outf = hsm_et_file_open(filename, "w", NULL);
	if (NULL != outf) {

		/* Erfolg im Rueckgabecode vermerken */
		exc = EXIT_SUCCESS;

		/* Falls Schreiben der Textzeile fehlschlaegt, Misserfolg */
		if (EOF == fputs("Dies ist ein Test-String", outf)) {
			exc = EXIT_FAILURE;
		}

		/* Falls Schreiben des Newline fehlschlaegt, Misserfolg */
		if (EOF == fputc('\n', outf)) {
			exc = EXIT_FAILURE;
		}

		/* Falls Schreiben der formatierten Ausgabe fehlschlaegt, Misserfolg */
		if (0 > fprintf(outf, "Der Wert der Variablen ist %d\n", i)) {
			exc = EXIT_FAILURE;
		}

		/* Falls Schliessen der Datei fehlschlaegt, Misserfolg */
		if (0 >= hsm_et_file_close(outf, filename, NULL)) {
			exc = EXIT_FAILURE;
		}
		outf = NULL;
	}
	exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 : */
