/** @file   ex031.c Demonstrationsprogramm zur Ressourcenfreigabe.

	ex031 Quelldatei Zieldatei
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#if HE_ON_WINDOWS
#define _CRT_SECURE_NO_WARNINGS 1
#endif

#include <stdio.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include "he-file.h"

/** Hauptprogramm.
	@param  argc    Anzahl der Kommandozeilen-Argumente.
	@param  argv    Kommandozeilen-Argumente.
	@return 0 bei Erfolg, alle anderen Werte weisen auf Fehler in.
*/
int
main(int argc, char *argv[])
{
	FILE      *infile     =   NULL;           /* Eingabedatei */
	FILE      *outfile    =   NULL;           /* Ausgabedatei */
	void      *puffer     =   NULL;           /* Puffer fuer Dateiinhalt */
	uintmax_t  filesize;                      /* Dateigroesse */
	size_t     rdbytes;                       /* Anzahl gelesene Bytes */
	size_t     wrbytes;                       /* Anzahl geschriebene Bytes */
	int        exc        =   EXIT_FAILURE;   /* Exit-Status-Code */
  
	if (3 == argc) {
		infile = fopen(argv[1], "rb");
		if (NULL != infile) {
			if (0 != hsm_et_file_pointer_size(&filesize,infile,argv[1],NULL)) {
				if (0 < filesize) {
					if ((uintmax_t)SIZE_MAX >= filesize) {
						puffer = calloc(1, filesize);
						if (NULL != puffer) {
							rdbytes = fread(puffer, 1, filesize, infile);
							if (filesize == (uintmax_t)rdbytes) {
								outfile = fopen(argv[2], "wb");
								if (NULL != outfile) {
									wrbytes = fwrite(puffer,1,rdbytes,outfile);
									if (wrbytes == rdbytes) {
										exc = EXIT_SUCCESS;
									}
									else {
										/* !!! Fehler beim Schreiben der Daten */
									}
									if (0 != fclose(outfile)) {
										exc = EXIT_FAILURE;
										/* !!! Fehler Schliessen Zieldatei */
									}
								}
								else {
									/* !!! Zieldatei nicht geoeffnet */
								}
							}
							else {
								/* !!! Nicht alle Bytes eingelesen */
							}
							free(puffer);
						}
						else {
							/* !!! Speicherallokation fehlgeschlagen */
						}
					}
					else {
						/* !!! Dateigroesse zu gross fuer size_t */
					}
				}
				else {
					/* !!! Quelldatei hat Groesse 0 */
				}
			}
			else {
				/* !!! Groesse der Quelldatei kann nicht bestimmt werden */
			}
			fclose(infile);
		}
		else {
			/* !!! Quelldatei kann nicht gelesen werden */
		}
	}
	else {
		/* !!! Falsche Anzahl an Kommandozeilenargumenten */
	}
	exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 : */
