/** @file   ex035.c Demonstrationsprogramm zur Ressourcenfreigabe.

	ex035 Quelldatei Zieldatei
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#if HE_ON_WINDOWS
#define _CRT_SECURE_NO_WARNINGS 1
#endif

#include <stdio.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include "he-file.h"

/** Hauptprogramm.
	@param  argc    Anzahl der Kommandozeilen-Argumente.
	@param  argv    Kommandozeilen-Argumente.
	@return 0 bei Erfolg, alle anderen Werte weisen auf Fehler in.
*/
int
main(int argc, char *argv[])
{
	FILE      *infile     =   NULL;           /* Eingabedatei */
	FILE      *outfile    =   NULL;           /* Ausgabedatei */
	void      *puffer     =   NULL;           /* Puffer fuer Dateiinhalt */
	uintmax_t  filesize;                      /* Dateigroesse */
	size_t     rdbytes;                       /* Anzahl gelesene Bytes */
	size_t     wrbytes;                       /* Anzahl geschriebene Bytes */
	int        exc        =   EXIT_FAILURE;   /* Exit-Status-Code */

	/* Anzahl an Kommandozeilenargumenten testen */
	if (3 != argc) {
		/* !!! Falsche Anzahl an Kommandozeilenargumenten */
		goto finished;
	}

	/* Eingabedatei oeffnen */
	infile = fopen(argv[1], "rb");
	if (NULL == infile) {
		/* !!! Quelldatei kann nicht gelesen werden */
		goto finished;
	}

	/* Versuchen, die Groesse der Quelldatei zu bestimmen */
	if (0 == hsm_et_file_pointer_size(&filesize, infile, argv[1], NULL)) {
		/* !!! Groesse der Quelldatei kann nicht bestimmt werden */
		goto finished;
	}

	/* Groessentest, Wert muss positiv sein */
	if (0 == filesize) {
		/* !!! Quelldatei hat Groesse 0 */
		goto finished;
	}

	/* Groessentest, Wert muss als size_t abbildbar sein */
	if ((uintmax_t)SIZE_MAX < filesize) {
		/* !!! Dateigroesse zu gross fuer size_t */
		goto finished;
	}

	/* Speicher dynamisch allozieren, Test */
	puffer = calloc(1, filesize);
	if (NULL == puffer) {
		/* !!! Speicherallokation fehlgeschlagen */
		goto finished;
	}

	/* Einlesen, Test ob vollstaendig eingelesen wurde. */
	rdbytes = fread(puffer, 1, filesize, infile);
	if ((uintmax_t)rdbytes < filesize) {
		/* !!! Nicht alle Bytes eingelesen */
		goto finished;
	}

	/* Ausgabedatei oeffnen */
	outfile = fopen(argv[2], "wb");
	if (NULL == outfile) {
		/* !!! Zieldatei nicht zum Schreiben geoeffnet */
		goto finished;
	}

	/* Dateiinhalt schreiben */
	wrbytes = fwrite(puffer, 1, rdbytes, outfile);
	if ((uintmax_t)wrbytes < rdbytes) {
		/* !!! Fehler beim Schreiben der Daten */
		goto finished;
	}

	/* Hier wurde alles erfolgreich abgearbeitet. */
	exc = EXIT_SUCCESS;

finished:
	if (NULL != outfile) {
		if (0 != fclose(outfile)) {
			/* !!! Fehler beim Schliessen der Zieldatei */
			exc = EXIT_FAILURE;
		}
	}
	if (NULL != puffer) {
		free(puffer);
	}
	if (NULL != infile) {
		fclose(infile);
	}
	exit(exc); return exc;
}
