/** @file   ex038.c Demo-Programm fuer Abfrage von Datum und Uhrzeit
					auf Windows-Systemen.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/** Hauptprogramm.
	@return 0 bei Erfolg, alle anderen Werte weisen auf Fehler hin.
*/
int
main(void)
{
	struct tm  buffer;                    /* Aktuelle Zeit zerlegt in Teile */
	__time64_t timer;                     /* Aktuelle Zeit als ein Wert */
	int        exc    =   EXIT_FAILURE;   /* Exit-Status-Code */
	
	/* Aktuelle Zeit ermitteln */
	if ((__time64_t)(-1) == _time64(&timer)) {
		/* !!! Holen der Uhrzeit ist fehlgeschlagen */
		goto finished;
	}

	/* Zeit in Komponenten zerlegen */
	if (0 != _localtime64_s(&buffer, &timer)) {
		/* !!! Fehler aufgetreten beim Zerlegen der Zeit */
		goto finished;
	}

	/* Datum und Uhrzeit ausgeben */
	printf(
		"Heute ist der %02d.%02d.%04d.\n",
		buffer.tm_mday,
		(1 + buffer.tm_mon),
		(1900 + buffer.tm_year)
	);
	printf(
		"Es ist        %02d:%02d:%02d Uhr.\n",
		buffer.tm_hour,
		buffer.tm_min,
		buffer.tm_sec
	);

	/* Erfolg vermerken */
	exc = EXIT_SUCCESS;

finished:
	exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 : */
