/* Testprogram zur Demonstration von Gleitkomma-Exceptions unter POSIX
   Mit z.B. folgenden Eingaben ausprobieren:

   1.5e200
   1.8e-200

*/
/* Copyright (C) 2015-2017 - HS Schmalkalden. All rights reserved. */

#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS 1
#endif

#include <stdio.h>

#include <fenv.h>
#include <float.h>
#include <math.h>

#pragma STDC FENV_ACCESS ON

/**     Flaechenberechnung fuer Quadrat.
		@param  a   Seitenlaenge des Quadrates.
		@return Flaecheninhalt des Quadrates bei Erfolg, -1.0 bei Fehler
		(negative Seitenlaenge).
*/
double
quadratflaeche(double a);


/**     Floating-Point-Ausnahmen.
*/
static  int flp_exc =   0;


/** Hauptprogramm.
	@return 0 bei Erfolg, 1 bei Fehler.
*/
int
main(void)
{
	int       exitcode = 1;   /* Exit-Code, 0 bei Erfolg, 1 bei Fehler */
	double    slaenge;        /* Seitenlaenge */
	double    flaeche;        /* Flaecheninhalt */

	printf("Bitte Seitenlaenge fuer Quadrat angeben: ");
	scanf("%lg", &slaenge);
	if (0.0 <= slaenge) {
		feclearexcept(FE_ALL_EXCEPT);
		flaeche = quadratflaeche(slaenge);
		flp_exc = fetestexcept(FE_ALL_EXCEPT & (~(FE_INEXACT)));
		if (0 != (FE_ALL_EXCEPT & flp_exc)) {
			printf("Es ist eine Gleitkomma-Ausnahme aufgetreten.\n");
			if (0 != (FE_DIVBYZERO & flp_exc)) {
				printf("Division durch 0!\n");
			}
			if (0 != (FE_INVALID & flp_exc)) {
				printf("Ungueltiger Operand!\n");
			}
			if (0 != (FE_OVERFLOW & flp_exc)) {
				printf("Zahlenbereichsueberlauf!\n");
			}
			if (0 != (FE_UNDERFLOW & flp_exc)) {
				printf("Zahlenbereichsunterlauf!\n");
			}
		}
		else {
			printf("Flaecheninhalt: %lg\n", flaeche);
		}
		exitcode = 0;
	}
	else {
		printf("FEHLER: Seitenlaenge darf nicht negativ sein!\n");
	}
	return exitcode;
}



double
quadratflaeche(double a)
{
	double    ergebnis    =   -1.0;

	if (0.0 <= a) {
		ergebnis = a * a;
	}
	return    ergebnis;
}

/* vim: set ai sw=4 ts=4 : */
