
#include "he-conf.h"
#include "he-error.h"
#include "he-bfio.h"



#if HAVE_ERRNO_H
#include <errno.h>
#endif

#if HAVE_STRING_H
#include <string.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_MALLOC_H
#include <malloc.h>
#endif

#if HAVE_STDINT_H
#include <stdint.h>
#endif


int
he_bfio_save_uint16(FILE *fp, unsigned short u16, int *ec)
{
	unsigned char	uc[4];
	int				back	= 0;

	if (NULL != fp) {
		uc[0] = (unsigned char)(u16        & 0x00FFU);
		uc[1] = (unsigned char)((u16 >> 8) & 0x00FFU);
		errno = 0;
		if (2 == fwrite(uc, 1, 2, fp)) {
			back = 1;
		}
		else {
			hsm_et_error_set(ec, HE_ERROR_SYSTEM);
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_save_uint32(FILE *fp, unsigned long u32, int *ec)
{

	unsigned char	uc[4];
	int				back	= 0;

	if (NULL != fp) {
		uc[0] = (unsigned char)(u32         & 0x00FFU);
		uc[1] = (unsigned char)((u32 >>  8) & 0x00FFU);
		uc[2] = (unsigned char)((u32 >> 16) & 0x00FFU);
		uc[3] = (unsigned char)((u32 >> 24) & 0x00FFU);
		errno = 0;
		if (4 == fwrite(uc, 1, 4, fp)) {
			back = 1;
		}
		else {
			hsm_et_error_set(ec, HE_ERROR_SYSTEM);
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_save_int16(FILE *fp, short i16, int *ec)
{
	return (he_bfio_save_uint16(fp, (unsigned short)i16, ec));
}



int
he_bfio_save_int32(FILE *fp, long i32, int *ec)
{
	return (he_bfio_save_uint32(fp, (unsigned long)i32, ec));
}



int
he_bfio_save_string(FILE *fp, const char *str, int *ec)
{
	size_t	sl;
	int		back    = 0;
	if ((NULL != fp) && (NULL != str)) {
		sl = strlen(str);
		if (SIZE_MAX > sl) {
			sl++;
			if (((size_t)0xFFFFU) >= sl) {
				if (0 < he_bfio_save_uint16(fp, (unsigned short)sl, ec)) {
					errno = 0;
					if (fwrite(str, 1, sl, fp) == sl) {
						back = 1;
					}
					else {
						hsm_et_error_set(ec, HE_ERROR_SYSTEM);
					}
				}
				else {
					/* Fehler schon behandelt */
				}
			}
			else {
				hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
			}
		}
		else {
			hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_restore_uint16(unsigned short *u16p, FILE *fp, int *ec)
{
	unsigned char	uc[4];
	int				back    = 0;
	unsigned short	us;

	if ((NULL != u16p) && (NULL != fp)) {
		errno = 0;
		if (2 == fread(uc, 1, 2, fp)) {
			us = ((unsigned short)(uc[0]) & 0x00FFU)
				 | ((((unsigned short)(uc[1])) <<  8) & 0xFF00U);
			*u16p = us;
			back = 1;
		}
		else {
			hsm_et_error_set(ec, HE_ERROR_SYSTEM);
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_restore_uint32(unsigned long *u32p, FILE *fp, int *ec)
{
	unsigned long	ul;
	unsigned char	uc[4];
	int				back    = 0;

	if ((NULL != u32p) && (NULL != fp)) {
		errno = 0;
		if (4 == fread(uc, 1, 4, fp)) {
			ul = ((unsigned long)(uc[0]) & 0x000000FFUL)
				 | ((((unsigned long)(uc[1])) <<  8) & 0x0000FF00UL)
				 | ((((unsigned long)(uc[2])) << 16) & 0x00FF0000UL)
				 | ((((unsigned long)(uc[3])) << 24) & 0xFF000000UL);
			*u32p = ul;
			back = 1;
		}
		else {
			hsm_et_error_set(ec, HE_ERROR_SYSTEM);
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_restore_int16(short *i16p, FILE *fp, int *ec)
{
	int				back	= 0;
	unsigned short	res		= 0U;

	if ((NULL != i16p) && (NULL != fp)) {
		back = he_bfio_restore_uint16(&res, fp, ec);
		if (0 != back) {
			*i16p = (short)res;
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_restore_int32(long *i32p, FILE *fp, int *ec)
{
	unsigned long	ul	= 0UL;
	int				back	= 0;
	if ((NULL != i32p) && (NULL != fp)) {
		back = he_bfio_restore_uint32(&ul, fp, ec);
		if (0 != back) {
			*i32p = (long)ul;
		}
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



int
he_bfio_restore_string(char **strp, FILE *fp, int *ec)
{
	char			*np		= NULL;
	int				 back	= 0;
	int				 oe;
	unsigned short	 sl		= 0U;

	if ((NULL != strp) && (NULL != fp)) {
		if (NULL == *strp) {
			if (0 != he_bfio_restore_uint16(&sl, fp, ec)) {
				if (0U < sl) {
					np = malloc((size_t)sl);
					if (NULL != np) {
						errno = 0;
						if (fread(np, 1, sl, fp) == sl) {
							if ('\0' == np[sl - 1]) {
								*strp = np;
								back  = 1;
							}
							else {
								hsm_et_error_set(ec, HE_ERROR_SYNTAX);
								oe = errno;
								free(np);
								errno = oe;
							}
						}
						else {
							hsm_et_error_set(ec, HE_ERROR_SYSTEM);
							oe = errno;
							free(np);
							errno = oe;
						}
					}
					else {
						hsm_et_error_set(ec, HE_ERROR_MEMORY);
					}
				}
				else {
					hsm_et_error_set(ec, HE_ERROR_SYNTAX);
				}
			}
		}
		else {
			hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
		}
	
	}
	else {
		hsm_et_error_set(ec, HE_ERROR_INVALID_ARGS);
	}
	return back;
}



/* vim: set ai sw=4 ts=4 : */
